/*  CSMA/CD (for collision-detection) bus arbitration algorithm
    for one bus and *two* senders. 
                  (1)^^^
    The model maintains mutual exclusion to the bus because after the 
    detection of the collision in A time units, 
    the collision will be detected by all senders in 2A=B time units.  
    The default process count in this template is 3, 
                                               (2)^
    with 1 bus and *2* senders.  
                 (3)^
    The process count cannot be overridden with option -Mk where 
    k is the number of *senders* without breaking the mutual exclusion.  
    The reason is that the bus needs to send out exactly *2* cd signals to 
                                                       (4)^
    the *two* senders.  
      (5)^^^
    How this template can be used ?
    
    There are the following specification files.  
    
    1. hcd2.3.risk-free: The risk condition is that either process 2 and 3, 
       for senders 1 and 2 respectively, 
       has been using the bus for more than B time units.  
       For example, to analyze the risk: 
    
        red -Tr hcd2.d hcd2.3.risk-free 
               (6)^
       
       The answer is no risk.  

    2. hcd2.3.risk-bad: The risk condition is that either process 2 and 3, 
       for senders 1 and 2 respectively, 
       has been using the bus for more than A time units.  

       For example, to analyze the risk with counter examples: 
    
        red -Tr -C hcd2.d hcd2.3.risk-bad 
                   (7)^
       
       The answer is that risk exists.  
       Option -C generates the counter example. 
*/ 
#define	A	26
#define	B	52
#define	LAMBDA	808

process count = 3; /* 1 is for bus, the others for senders. */ 
          /*(8) ^ */
local clock x;
global synchronizer begin, end, cd, busy; 

mode bus_idle (true) { 
/*1*/	when ?begin (true)  may x= 0; goto bus_active; /* 1 */
}

mode bus_active (true) { 
/*2*/	when ?end (true)  may x= 0; goto bus_idle; /* 2 */
/*3*/	when !busy (x >= A) may ; /* 3 */
/*4*/	when ?begin (x < A)  may x= 0; goto bus_collision; /* 4 */
}

mode bus_collision (x < A) { 
/*5*/	when !(#PS-1)cd (x < A)  may x= 0; goto bus_idle; /* 5 */
       /*(9) ^^^^^^^ */
}


mode sender_wait (true) {
/*6*/	when !begin (true) may x= 0; goto sender_transm; /* 6 */
/*7*/	when ?cd (true) may x= 0; /* 7 */
/*8*/	when ?cd (true) may x= 0; goto sender_retry; /* 8 */
/*9*/	when ?busy (true) may x= 0; goto sender_retry; /* 9 */
}  

mode sender_transm (x <= LAMBDA) { 
/*10*/	when !end (x==LAMBDA) may x= 0; goto sender_wait; /* 10 */
/*11*/	when ?cd (x<B) may x= 0; goto sender_retry; /* 11 */
} 

mode sender_retry (x < B) {
/*12*/	when !begin (x < B) may x= 0; goto sender_transm; /* 12 */ 
/*13*/	when ?busy (true) may x= 0; /* 13 */
/*14*/	when ?cd (x < B) may x= 0; /* 14 */
}



initially 
    bus_idle[1] && x[1] == 0 
&&  forall i:2..#PS, (sender_wait[i] and x[i] == 0); 

